# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from openstack.tests.unit import test_proxy_base

from otcextensions.sdk.vlb.v3 import _proxy
from otcextensions.sdk.vlb.v3 import availability_zone
from otcextensions.sdk.vlb.v3 import certificate
from otcextensions.sdk.vlb.v3 import flavor
from otcextensions.sdk.vlb.v3 import health_monitor
from otcextensions.sdk.vlb.v3 import l7_policy
from otcextensions.sdk.vlb.v3 import l7_rule
from otcextensions.sdk.vlb.v3 import listener
from otcextensions.sdk.vlb.v3 import load_balancer
from otcextensions.sdk.vlb.v3 import member
from otcextensions.sdk.vlb.v3 import pool


class TestVlbProxy(test_proxy_base.TestProxyBase):
    def setUp(self):
        super(TestVlbProxy, self).setUp()
        self.proxy = _proxy.Proxy(self.session)


class TestVlbLoadBalancer(TestVlbProxy):
    def test_load_balancer_create(self):
        self.verify_create(
            self.proxy.create_load_balancer,
            load_balancer.LoadBalancer,
            method_kwargs={
                'az_list': ['az'],
                'name': 'name',
                'description': 'description',
                'guaranteed': True,
                'provider': 'vlb',
                'ip_target_enable': True
            },
            expected_kwargs={
                'az_list': ['az'],
                'name': 'name',
                'description': 'description',
                'guaranteed': True,
                'provider': 'vlb',
                'ip_target_enable': True
            }
        )

    def test_load_balancer_delete(self):
        self.verify_delete(self.proxy.delete_load_balancer,
                           load_balancer.LoadBalancer, True)

    def test_load_balancer_get(self):
        self.verify_get(
            self.proxy.get_load_balancer,
            load_balancer.LoadBalancer
        )

    def test_load_balancers(self):
        self.verify_list(
            self.proxy.load_balancers,
            load_balancer.LoadBalancer
        )

    def test_load_balancer_find(self):
        self.verify_find(
            self.proxy.find_load_balancer,
            load_balancer.LoadBalancer
        )

    def test_load_balancer_update(self):
        self.verify_update(
            self.proxy.update_load_balancer,
            load_balancer.LoadBalancer
        )


class TestVlbListener(TestVlbProxy):
    def test_listener_create(self):
        self.verify_create(
            self.proxy.create_listener,
            listener.Listener,
            method_kwargs={
                'name': 'name',
                'protocol_port': 80,
                'protocol': 'TCP',
            },
            expected_kwargs={
                'name': 'name',
                'protocol_port': 80,
                'protocol': 'TCP',
            }
        )

    def test_listener_delete(self):
        self.verify_delete(self.proxy.delete_listener,
                           listener.Listener, True)

    def test_listener_get(self):
        self.verify_get(
            self.proxy.get_listener,
            listener.Listener
        )

    def test_listeners(self):
        self.verify_list(
            self.proxy.listeners,
            listener.Listener
        )

    def test_listener_find(self):
        self.verify_find(
            self.proxy.find_listener,
            listener.Listener
        )

    def test_listener_update(self):
        self.verify_update(
            self.proxy.update_listener,
            listener.Listener
        )


class TestVlbCertificate(TestVlbProxy):
    def test_certificate_create(self):
        self.verify_create(
            self.proxy.create_certificate,
            certificate.Certificate,
            method_kwargs={
                'private_key': '_private_key',
                'certificate': '_certificate',
                'name': 'name'
            },
            expected_kwargs={
                'private_key': '_private_key',
                'certificate': '_certificate',
                'name': 'name'
            }
        )

    def test_certificate_delete(self):
        self.verify_delete(self.proxy.delete_certificate,
                           certificate.Certificate, True)

    def test_certificate_get(self):
        self.verify_get(
            self.proxy.get_certificate,
            certificate.Certificate
        )

    def test_certificates(self):
        self.verify_list(
            self.proxy.certificates,
            certificate.Certificate
        )

    def test_certificate_find(self):
        self.verify_find(
            self.proxy.find_certificate,
            certificate.Certificate
        )

    def test_certificate_update(self):
        self.verify_update(
            self.proxy.update_certificate,
            certificate.Certificate
        )


class TestVlbAvailabilityZone(TestVlbProxy):
    def test_availability_zones(self):
        self.verify_list(
            self.proxy.availability_zones,
            availability_zone.AvailabilityZone
        )


class TestVlbFlavor(TestVlbProxy):
    def test_flavors(self):
        self.verify_list(
            self.proxy.flavors,
            flavor.Flavor
        )

    def test_flavor_get(self):
        self.verify_get(
            self.proxy.get_flavor,
            flavor.Flavor
        )

    def test_flavor_find(self):
        self.verify_find(
            self.proxy.find_flavor,
            flavor.Flavor
        )


class TestVlbPool(TestVlbProxy):
    def test_pool_create(self):
        self.verify_create(
            self.proxy.create_pool,
            pool.Pool,
            method_kwargs={
                'lb_algorithm': 'ROUND_ROBIN',
                'name': 'name',
                'protocol': 'TCP',
            },
            expected_kwargs={
                'lb_algorithm': 'ROUND_ROBIN',
                'name': 'name',
                'protocol': 'TCP',
            }
        )

    def test_pool_delete(self):
        self.verify_delete(self.proxy.delete_pool,
                           pool.Pool, True)

    def test_pool_get(self):
        self.verify_get(
            self.proxy.get_pool,
            pool.Pool
        )

    def test_pools(self):
        self.verify_list(
            self.proxy.pools,
            pool.Pool
        )

    def test_pool_find(self):
        self.verify_find(
            self.proxy.find_pool,
            pool.Pool
        )

    def test_pool_update(self):
        self.verify_update(
            self.proxy.update_pool,
            pool.Pool
        )


class TestVlbMember(TestVlbProxy):
    def test_member_create(self):
        self.verify_create(
            self.proxy.create_member,
            member.Member,
            method_kwargs={
                'pool': 'pool',
                'name': 'name',
                'protocol_port': 8080,
            },
            expected_kwargs={
                'pool_id': 'pool',
                'name': 'name',
                'protocol_port': 8080,
            }
        )

    def test_member_delete(self):
        self.verify_delete(
            self.proxy.delete_member,
            member.Member,
            True,
            method_kwargs={
                'pool': 'pool'
            },
            expected_kwargs={
                'pool_id': 'pool',
                'ignore_missing': True
            }
        )

    def test_member_get(self):
        self.verify_get(
            self.proxy.get_member,
            member.Member,
            method_kwargs={
                'pool': 'pool'
            },
            expected_kwargs={
                'pool_id': 'pool',
            }
        )

    def test_members(self):
        self.verify_list(
            self.proxy.members,
            member.Member,
            method_kwargs={
                'pool': 'pool'
            },
            expected_kwargs={
                'pool_id': 'pool',
            }
        )

    def test_member_find(self):
        self.verify_find(
            self.proxy.find_member,
            member.Member,
            method_kwargs={
                'pool': 'pool'
            },
            expected_kwargs={
                'pool_id': 'pool',
            }
        )

    def test_member_update(self):
        self.verify_update(
            self.proxy.update_member,
            member.Member,
            method_kwargs={
                'pool': 'pool'
            },
            expected_kwargs={
                'pool_id': 'pool',
            }
        )


class TestVlbHealthMonitor(TestVlbProxy):
    def test_health_monitor_create(self):
        self.verify_create(
            self.proxy.create_health_monitor,
            health_monitor.HealthMonitor,
            method_kwargs={
                'type': 'TCP',
                'timeout': 3,
                'delay': 5,
                'max_retries': 3,
                'admin_state_up': True,
                'monitor_port': 8080,
            },
            expected_kwargs={
                'type': 'TCP',
                'timeout': 3,
                'delay': 5,
                'max_retries': 3,
                'admin_state_up': True,
                'monitor_port': 8080,
            }
        )

    def test_health_monitor_delete(self):
        self.verify_delete(self.proxy.delete_health_monitor,
                           health_monitor.HealthMonitor, True)

    def test_health_monitor_get(self):
        self.verify_get(
            self.proxy.get_health_monitor,
            health_monitor.HealthMonitor
        )

    def test_health_monitors(self):
        self.verify_list(
            self.proxy.health_monitors,
            health_monitor.HealthMonitor
        )

    def test_health_monitor_find(self):
        self.verify_find(
            self.proxy.find_health_monitor,
            health_monitor.HealthMonitor
        )

    def test_health_monitor_update(self):
        self.verify_update(
            self.proxy.update_health_monitor,
            health_monitor.HealthMonitor
        )


class TestVlbL7Policy(TestVlbProxy):
    def test_l7_policy_create(self):
        self.verify_create(
            self.proxy.create_l7_policy,
            l7_policy.L7Policy,
            method_kwargs={
                'action': 'action',
                'name': 'name',
                'redirect_listener_id': 'id',
            },
            expected_kwargs={
                'action': 'action',
                'name': 'name',
                'redirect_listener_id': 'id',
            }
        )

    def test_l7_policy_delete(self):
        self.verify_delete(self.proxy.delete_l7_policy,
                           l7_policy.L7Policy, True)

    def test_l7_policy_get(self):
        self.verify_get(
            self.proxy.get_l7_policy,
            l7_policy.L7Policy
        )

    def test_l7_policies(self):
        self.verify_list(
            self.proxy.l7_policies,
            l7_policy.L7Policy
        )

    def test_l7_policy_find(self):
        self.verify_find(
            self.proxy.find_l7_policy,
            l7_policy.L7Policy
        )

    def test_l7_policy_update(self):
        self.verify_update(
            self.proxy.update_l7_policy,
            l7_policy.L7Policy
        )


class TestVlbL7Rule(TestVlbProxy):
    def test_l7_rule_create(self):
        self.verify_create(
            self.proxy.create_l7_rule,
            l7_rule.L7Rule,
            method_kwargs={
                'l7_policy': 'l7_policy',
            },
            expected_kwargs={
                'l7policy_id': 'l7_policy',
            }
        )

    def test_l7_rule_delete(self):
        self.verify_delete(
            self.proxy.delete_l7_rule,
            l7_rule.L7Rule,
            True,
            method_kwargs={
                'l7_policy': 'l7_policy'
            },
            expected_kwargs={
                'l7policy_id': 'l7_policy',
            }
        )

    def test_l7_rule_get(self):
        self.verify_get(
            self.proxy.get_l7_rule,
            l7_rule.L7Rule,
            method_kwargs={
                'l7_policy': 'l7_policy'
            },
            expected_kwargs={
                'l7policy_id': 'l7_policy',
            }
        )

    def test_l7_rules(self):
        self.verify_list(
            self.proxy.l7_rules,
            l7_rule.L7Rule,
            method_kwargs={
                'l7_policy': 'l7_policy'
            },
            expected_kwargs={
                'l7policy_id': 'l7_policy',
            }
        )

    def test_l7_rule_find(self):
        self.verify_find(
            self.proxy.find_l7_rule,
            l7_rule.L7Rule,
            method_kwargs={
                'l7_policy': 'l7_policy'
            },
            expected_kwargs={
                'l7policy_id': 'l7_policy',
            }
        )

    def test_l7_rule_update(self):
        self.verify_update(
            self.proxy.update_l7_rule,
            l7_rule.L7Rule,
            method_kwargs={
                'l7_policy': 'l7_policy'
            },
            expected_kwargs={
                'l7policy_id': 'l7_policy',
            }
        )



#!/usr/bin/env python3
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
"""
Create CSS Cluster (opensearch)
"""

import openstack

openstack.enable_logging(True)
conn = openstack.connect(cloud='otc')


attrs = {
    'name': 'opensearch_test',
    'instanceNum': 3,
    'datastore': {
        'type': 'opensearch',
        'version': '1.3.6',
    },
    'instance': {
        'availability_zone': 'eu-de-01',
        'flavorRef': 'css.xlarge.2',
        'volume': {
            'volume_type': 'COMMON',
            'size': 100,
        },
        'nics': {
            'vpcId': 'VPC-ID',
            'netId': 'Network-ID',
            'securityGroupId': 'SecurityGroup-ID',
        },
    },
    'httpsEnable': 'false',
    'diskEncryption': {
        'systemEncrypted': '0',
    },
    'tags': [
        {'key': 'key0', 'value': 'value0'},
        {'key': 'key1', 'value': 'value1'},
    ],
    'backupStrategy': {
        'period': '00:00 GMT+01:00',
        'prefix': 'backup',
        'keepday': 1,
        'bucket': 'css-backup-1663481103064',
        'agency': 'test_agency',
        'basePath': 'css',
    },
}
result = conn.css.create_cluster(**attrs)
conn.css.wait_for_cluster(result)
cluster = conn.css.get_cluster(result)
print(cluster)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
import os
import uuid

# from openstack import resource

from otcextensions.tests.functional.sdk.swr import TestSwr


class TestOrganization(TestSwr):

    def setUp(self):
        super(TestOrganization, self).setUp()

        self.org_name = "sdk-swr-org-" + uuid.uuid4().hex
        self.org = self.client.create_organization(
            namespace=self.org_name
        )
        if os.getenv("OS_SWR_PERMISSIONS_RUN"):
            self.permission = [
                {
                    "user_id": "5a23ecb3999b458d92d51d524bb7fb4b",
                    "user_name": "pgubina",
                    "user_auth": 1
                }
            ]
            self.org_perm = self.client.create_organization_permissions(
                namespace=self.org_name, permissions=self.permission
            )
        self.addCleanup(self.conn.swr.delete_organization, self.org_name)

    def test_list_organizations(self):
        query = {}
        orgs = list(self.client.organizations(**query))
        self.assertGreaterEqual(len(orgs), 0)

    def test_get_organization(self):
        o = self.client.get_organization(self.org.namespace)
        self.assertEqual(self.org.namespace, o.name)
        self.assertEqual(7, o.user_auth)

    def test_find_organization(self):
        o = self.client.find_organization(self.org_name)
        self.assertEqual(self.org.namespace, o.name)
        self.assertEqual(7, o.user_auth)

    def test_organization_permissions(self):
        o = list(self.client.organization_permissions(self.org.namespace))
        self.assertEqual(self.org.namespace, o[0].name)

    def test_update_organization_permissions(self):
        if os.getenv("OS_SWR_PERMISSIONS_RUN"):
            o = self.client.update_organization_permissions(
                namespace=self.org_name, permissions=[
                    {
                        "user_id": "5a23ecb3999b458d92d51d524bb7fb4b",
                        "user_name": "pgubina",
                        "user_auth": 3
                    }
                ]
            )
            self.assertEqual(3, o.permissions[0].user_auth)

    def test_delete_organization_permissions(self):
        if os.getenv("OS_SWR_PERMISSIONS_RUN"):
            orgs = self.client.delete_organization_permissions(
                self.org.namespace,
                [self.org_perm.permissions[0].user_id]
            )
            self.assertEqual(orgs, None)



#   Licensed under the Apache License, Version 2.0 (the "License"); you may
#   not use this file except in compliance with the License. You may obtain
#   a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#   WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#   License for the specific language governing permissions and limitations
#   under the License.
#
"""ModelArts v1 trainingjob version action implementations"""
import logging

from osc_lib import utils
from osc_lib.command import command

from otcextensions.common import sdk_utils
from otcextensions.i18n import _

LOG = logging.getLogger(__name__)


def _get_columns(item):
    column_map = {}
    hidden = ["location"]
    return sdk_utils.get_osc_show_columns_for_sdk_resource(
        item, column_map, hidden
    )


class TrainingjobVersionLogs(command.ShowOne):
    _description = _(
        "Query detailed information about training job logs by row"
    )

    def get_parser(self, prog_name):
        parser = super(TrainingjobVersionLogs, self).get_parser(prog_name)
        parser.add_argument(
            "--job_id",
            metavar="<job_id>",
            required=True,
            type=int,
            help=_("ID of a training job"),
        )
        parser.add_argument(
            "--version_id",
            metavar="<version_id>",
            required=True,
            type=int,
            help=_("Version ID of a training job"),
        )
        parser.add_argument(
            "--base_line",
            metavar="<base_line>",
            required=False,
            type=str,
            help=_(
                "Base line of the log, which is obtained from an API response"
            ),
        )
        parser.add_argument(
            "--lines",
            metavar="<lines>",
            required=False,
            type=int,
            help=_("Length of the obtained log"),
        )
        parser.add_argument(
            "--log_file",
            metavar="<log_file>",
            required=True,
            type=str,
            help=_("Name of the log file to be viewed"),
        )
        parser.add_argument(
            "--order",
            metavar="<order>",
            required=False,
            type=str,
            help=_("Log query directiondesc: Query next records"),
        )
        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.modelartsv1
        attrs = {}
        args_list = [
            "project_id",
            "job_id",
            "version_id",
            "base_line",
            "lines",
            "log_file",
            "order",
        ]
        for arg in args_list:
            val = getattr(parsed_args, arg)
            if val:
                attrs[arg] = val

        data = client.list_trainingjob_version_logs(**attrs)
        display_columns, columns = _get_columns(data)
        data = utils.get_item_properties(data)

        return display_columns, data



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from openstack.tests.unit import test_proxy_base

from otcextensions.sdk.kms.v1 import _proxy
from otcextensions.sdk.kms.v1 import data_key as _data_key
from otcextensions.sdk.kms.v1 import key as _key
from otcextensions.sdk.kms.v1 import misc as _misc


class TestKmsProxy(test_proxy_base.TestProxyBase):
    def setUp(self):
        super(TestKmsProxy, self).setUp()
        self.proxy = _proxy.Proxy(self.session)


class TestKmsKeys(TestKmsProxy):

    def test_list(self):
        self.verify_list(
            self.proxy.keys, _key.Key,
            # mock_method='openstack.proxy.Proxy._list',
            method_kwargs={
                'some_arg': 'arg_value',
            },
            expected_kwargs={
                'some_arg': 'arg_value',
            }
        )

    def test_get(self):
        self.verify_get(
            self.proxy.get_key,
            _key.Key,
            # mock_method='otcextensions.sdk.sdk_proxy.Proxy._get',
            expected_kwargs={
            }
        )

    def test_create(self):
        self.verify_create(
            self.proxy.create_key, _key.Key,
            mock_method='openstack.proxy.Proxy._create',
            method_kwargs={
                'instance': 'test',
                'name': 'some_name'
            },
            expected_kwargs={
                'prepend_key': False,
                'instance': 'test',
                'name': 'some_name'
            }
        )

    def test_enable(self):
        self._verify(
            'otcextensions.sdk.kms.v1.key.Key.enable',
            self.proxy.enable_key,
            method_args=['INSTANCE'],
            expected_args=[self.proxy]
        )

    def test_disable(self):
        self._verify(
            'otcextensions.sdk.kms.v1.key.Key.disable',
            self.proxy.disable_key,
            method_args=['INSTANCE'],
            expected_args=[self.proxy]
        )

    def test_schedule_deletion(self):
        self._verify(
            'otcextensions.sdk.kms.v1.key.Key.schedule_deletion',
            self.proxy.schedule_key_deletion,
            method_args=['INSTANCE', 3],
            expected_args=[self.proxy, 3]
        )

    def test_cancel_deletion(self):
        self._verify(
            'otcextensions.sdk.kms.v1.key.Key.cancel_deletion',
            self.proxy.cancel_key_deletion,
            method_args=['INSTANCE'],
            expected_args=[self.proxy]
        )


class TestKmsDataKeys(TestKmsProxy):

    def test_create(self):
        self.verify_create(
            self.proxy.create_datakey, _data_key.DataKey,
            mock_method='openstack.proxy.Proxy._create',
            method_args=['CMK'],
            method_kwargs={
                'name': 'some_name'
            },
            expected_kwargs={
                'prepend_key': False,
                'name': 'some_name',
                'key_id': 'CMK'
            },
            expected_args=[]
        )

    def test_create_no_plain(self):
        self._verify(
            'otcextensions.sdk.kms.v1.data_key.DataKey.create_wo_plain',
            self.proxy.create_datakey_wo_plain,
            method_args=['INSTANCE'],
            expected_args=[self.proxy],
            expected_kwargs={
                'prepend_key': False,
            }
        )

    # def test_encrypt(self):
    #     self._verify(
    #         'otcextensions.sdk.kms.v1.data_key.DataKey.encrypt',
    #         self.proxy.encrypt_datakey,
    #         method_args=['INSTANCE'],
    #         expected_args=[self.proxy],
    #     )
    #
    # def test_decrypt(self):
    #     self._verify(
    #         'otcextensions.sdk.kms.v1.data_key.DataKey.decrypt',
    #         self.proxy.decrypt_datakey,
    #         method_args=['INSTANCE'],
    #         expected_args=[self.proxy],
    #     )


class TestKmsRandom(TestKmsProxy):

    def test_create(self):
        self.verify_create(
            self.proxy.generate_random, _misc.Random,
            mock_method='openstack.proxy.Proxy._create',
            method_kwargs={
                'random_data_length': 100
            },
            expected_kwargs={
                'prepend_key': False,
                'random_data_length': 100
            }
        )


class TestKmsInstanceNum(TestKmsProxy):

    def test_get(self):
        self._verify(
            'otcextensions.sdk.kms.v1.misc.InstanceNumber.fetch',
            self.proxy.get_instance_number,
            method_args=[],
            expected_args=[self.proxy],
        )


class TestKmsQuota(TestKmsProxy):

    def test_quotas(self):
        self._verify(
            'otcextensions.sdk.kms.v1.misc.Quota.list',
            self.proxy.quotas,
            method_args=[],
            expected_args=[self.proxy],
        )



#   Licensed under the Apache License, Version 2.0 (the "License"); you may
#   not use this file except in compliance with the License. You may obtain
#   a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#   WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#   License for the specific language governing permissions and limitations
#   under the License.
import mock

from otcextensions.osclient.deh.v1 import host
from otcextensions.tests.unit.osclient.deh.v1 import fakes


class TestListHost(fakes.TestDeH):

    objects = fakes.FakeHost.create_multiple(3)

    columns = (
        'id', 'name', 'auto_placement', 'availability_zone',
        'available_vcpus', 'available_memory'
    )

    data = []

    for s in objects:
        data.append(fakes.gen_data(s, columns))

    def setUp(self):
        super(TestListHost, self).setUp()

        self.cmd = host.ListHost(self.app, None)

        self.client.hosts = mock.Mock()
        self.client.api_mock = self.client.hosts

    def test_default(self):
        arglist = [
        ]

        verifylist = [
        ]

        # Verify cm is triggereg with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Set the response
        self.client.api_mock.side_effect = [
            self.objects
        ]

        # Trigger the action
        columns, data = self.cmd.take_action(parsed_args)

        self.client.api_mock.assert_called_once_with(
        )

        self.assertEqual(self.columns, columns)
        self.assertEqual(self.data, list(data))

    def test_default_query(self):
        arglist = [
            '--id', 'some_id',
            '--name', 'some_name',
            '--host_type', 'some_type',
            '--host_type_name', 'some_type_name',
            '--flavor', 'some_flavor',
            '--status', 'available',
            '--tenant', 'all',
            '--availability_zone', 'some_az',
            '--limit', '1',
            '--marker', 'some_marker',
            '--changes_since', '2200-01-01T00:00:00+00:00'
        ]

        verifylist = [
            ('id', 'some_id'),
            ('name', 'some_name'),
            ('host_type', 'some_type'),
            ('host_type_name', 'some_type_name'),
            ('flavor', 'some_flavor'),
            ('status', 'available'),
            ('tenant', 'all'),
            ('availability_zone', 'some_az'),
            ('limit', 1),
            ('marker', 'some_marker'),
            ('changes_since', '2200-01-01T00:00:00+00:00')
        ]

        # Verify cm is triggereg with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Set the response
        self.client.api_mock.side_effect = [
            self.objects
        ]

        # Trigger the action
        columns, data = self.cmd.take_action(parsed_args)

        self.client.api_mock.assert_called_once_with(
            availability_zone='some_az',
            changes_since='2200-01-01T00:00:00+00:00',
            flavor='some_flavor',
            host_type='some_type',
            host_type_name='some_type_name',
            id='some_id',
            limit=1,
            marker='some_marker',
            name='some_name',
            status='available',
            tenant='all'
        )

        self.assertEqual(self.columns, columns)
        self.assertEqual(self.data, list(data))


class TestShowHost(fakes.TestDeH):

    _data = fakes.FakeHost.create_one()

    columns = (
        'allocated_at', 'auto_placement', 'availability_zone',
        'available_memory', 'available_vcpus', 'host_properties', 'id',
        'instance_total', 'name', 'project_id', 'released_at', 'status', 'tags'
    )

    data = fakes.gen_data(_data, columns)

    def setUp(self):
        super(TestShowHost, self).setUp()

        self.cmd = host.ShowHost(self.app, None)

        self.client.find_host = mock.Mock()
        self.client.api_mock = self.client.find_host
        self._data.fetch_tags = mock.Mock()

    def test_default(self):
        arglist = [
            'host'
        ]

        verifylist = [
            ('host', 'host')
        ]

        # Verify cm is triggereg with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Set the response
        self.client.api_mock.side_effect = [
            self._data
        ]

        # Trigger the action
        columns, data = self.cmd.take_action(parsed_args)

        self.client.api_mock.assert_called_once_with(
            'host'
        )

        self.assertEqual(self.columns, columns)
        self.assertItemEqual(self.data, data)


class TestCreateHost(fakes.TestDeH):

    _data = fakes.FakeHost.create_one()

    columns = (
        'id'
    )

    _data.dedicated_host_ids = [_data.id]

    data = []

    for s in _data.dedicated_host_ids:
        data.append(fakes.gen_data(s, columns))

    def setUp(self):
        super(TestCreateHost, self).setUp()

        self.cmd = host.CreateHost(self.app, None)

        self.client.create_host = mock.Mock()
        self.client.api_mock = self.client.create_host

    def test_create(self):
        arglist = [
            '--name', 'name',
            '--auto_placement',
            '--availability_zone', 'az1',
            '--host_type', 'type',
            '--quantity', '1',
        ]

        verifylist = [
            ('name', 'name'),
            ('auto_placement', 'on'),
            ('availability_zone', 'az1'),
            ('host_type', 'type'),
            ('quantity', 1),
        ]

        # Verify cm is triggereg with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Set the response
        self.client.api_mock.side_effect = [
            self._data
        ]

        # Trigger the action
        columns, data = self.cmd.take_action(parsed_args)

        self.client.api_mock.assert_called_once_with(
            auto_placement='on', availability_zone='az1',
            host_type='type', name='name', quantity=1
        )

        self.assertEqual(self.columns, columns)
        self.assertEqual(self.data, list(data))

    def test_create_default_placement(self):
        arglist = [
            '--name', 'name',
            '--availability_zone', 'az1',
        ]

        verifylist = [
            ('name', 'name'),
            ('auto_placement', 'on'),
            ('availability_zone', 'az1'),
            ('quantity', 1),
        ]

        # Verify cm is triggereg with default parameters
        self.check_parser(self.cmd, arglist, verifylist)

    def test_create_no_placement(self):
        arglist = [
            '--name', 'name',
            '--no-auto_placement',
            '--availability_zone', 'az1',
        ]

        verifylist = [
            ('name', 'name'),
            ('auto_placement', 'off'),
            ('availability_zone', 'az1'),
            ('quantity', 1),
        ]

        # Verify cm is triggereg with default parameters
        self.check_parser(self.cmd, arglist, verifylist)


class TestSetHost(fakes.TestDeH):

    _fake = fakes.FakeHost.create_one()

    columns = (
        'allocated_at', 'auto_placement', 'availability_zone',
        'available_memory', 'available_vcpus', 'host_properties', 'id',
        'instance_total', 'name', 'project_id', 'released_at', 'status', 'tags'
    )

    data = fakes.gen_data(_fake, columns)

    def setUp(self):
        super(TestSetHost, self).setUp()

        self.cmd = host.SetHost(self.app, None)

        self.client.update_host = mock.Mock()
        self.client.find_host = mock.Mock()
        self.client.api_mock = self.client.update_host
        self._fake.fetch_tags = mock.Mock()
        self._fake.add_tags = mock.Mock()

    def test_update(self):
        arglist = [
            'zn',
            '--name', 'name',
            '--auto_placement'
        ]

        verifylist = [
            ('host', 'zn'),
            ('name', 'name'),
            ('auto_placement', 'on')
        ]

        # Verify cm is triggereg with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Set the response
        self.client.api_mock.side_effect = [
            self._fake
        ]
        self.client.find_host.side_effect = [
            self._fake
        ]

        # Trigger the action
        columns, data = self.cmd.take_action(parsed_args)

        self.client.find_host.assert_called_once_with(
            'zn',
            ignore_missing=False
        )

        self.client.api_mock.assert_called_once_with(
            host=self._fake,
            name='name',
            auto_placement='on',
        )

        self.assertEqual(self.columns, columns)
        self.assertItemEqual(self.data, data)

    def test_update_disable_placement(self):
        arglist = [
            'zn',
            '--name', 'name',
            '--no-auto_placement'
        ]

        verifylist = [
            ('host', 'zn'),
            ('name', 'name'),
            ('auto_placement', 'off'),
        ]

        # Verify cm is triggereg with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Set the response
        self.client.api_mock.side_effect = [
            self._fake
        ]
        self.client.find_host.side_effect = [
            self._fake
        ]

        # Trigger the action
        columns, data = self.cmd.take_action(parsed_args)

        self.client.find_host.assert_called_once_with(
            'zn',
            ignore_missing=False
        )

        self.client.api_mock.assert_called_once_with(
            host=self._fake,
            name='name',
            auto_placement='off',
        )

        self.assertEqual(self.columns, columns)
        self.assertItemEqual(self.data, data)

    def test_add_tags(self):
        arglist = [
            'zn',
            '--tag', 'a=b',
            '--tag', 'c=d'
        ]

        verifylist = [
            ('host', 'zn'),
            ('tag', ['a=b', 'c=d'])
        ]

        # Verify cm is triggereg with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Set the response
        self.client.find_host.side_effect = [
            self._fake
        ]
        self._fake.add_tags.side_effect = [
            self._fake
        ]

        # Trigger the action
        columns, data = self.cmd.take_action(parsed_args)

        self.client.find_host.assert_called_once_with(
            'zn',
            ignore_missing=False
        )

        self.client.api_mock.assert_not_called()

        self._fake.add_tags.assert_called_once_with(
            mock.ANY,
            [{'key': 'a', 'value': 'b'},
             {'key': 'c', 'value': 'd'}]
        )

        self.assertEqual(self.columns, columns)
        self.assertItemEqual(self.data, data)


class TestUnsetHost(fakes.TestDeH):

    _fake = fakes.FakeHost.create_one()

    columns = (
        'allocated_at', 'auto_placement', 'availability_zone',
        'available_memory', 'available_vcpus', 'host_properties', 'id',
        'instance_total', 'name', 'project_id', 'released_at', 'status', 'tags'
    )

    data = fakes.gen_data(_fake, columns)

    def setUp(self):
        super(TestUnsetHost, self).setUp()

        self.cmd = host.UnsetHost(self.app, None)

        self.client.update_host = mock.Mock()
        self.client.find_host = mock.Mock()
        self.client.api_mock = self.client.update_host
        self._fake.fetch_tags = mock.Mock()
        self._fake.remove_tags = mock.Mock()

    def test_remove_tags_with_value(self):
        tag = self._fake.tags[0]
        tag_str = '{key}={value}'.format(
            key=tag['key'],
            value=tag['value'])
        arglist = [
            'zn',
            '--tag', tag_str,
            '--tag', 'e'
        ]

        verifylist = [
            ('host', 'zn'),
            ('tag', [tag_str, 'e'])
        ]

        # Verify cm is triggereg with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Set the response
        self.client.find_host.side_effect = [
            self._fake
        ]
        self._fake.fetch_tags.side_effect = [
            self._fake
        ]
        self._fake.remove_tags.side_effect = [
            self._fake
        ]

        # Trigger the action
        columns, data = self.cmd.take_action(parsed_args)

        self.client.find_host.assert_called_once_with(
            'zn',
            ignore_missing=False
        )

        self.client.api_mock.assert_not_called()

        self._fake.remove_tags.assert_called_once_with(
            mock.ANY,
            [tag]
        )

        self.assertEqual(self.columns, columns)
        self.assertItemEqual(self.data, data)

    def test_remove_tags_without_value(self):
        tag = self._fake.tags[0]
        arglist = [
            'zn',
            '--tag', tag['key'],
            '--tag', 'e'
        ]

        verifylist = [
            ('host', 'zn'),
            ('tag', [tag['key'], 'e'])
        ]

        # Verify cm is triggereg with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Set the response
        self.client.find_host.side_effect = [
            self._fake
        ]
        self._fake.fetch_tags.side_effect = [
            self._fake
        ]
        self._fake.remove_tags.side_effect = [
            self._fake
        ]

        # Trigger the action
        columns, data = self.cmd.take_action(parsed_args)

        self.client.find_host.assert_called_once_with(
            'zn',
            ignore_missing=False
        )

        self.client.api_mock.assert_not_called()

        self._fake.remove_tags.assert_called_once_with(
            mock.ANY,
            [tag]
        )

        self.assertEqual(self.columns, columns)
        self.assertItemEqual(self.data, data)


class TestDeleteHost(fakes.TestDeH):

    def setUp(self):
        super(TestDeleteHost, self).setUp()

        self.cmd = host.DeleteHost(self.app, None)

        self.client.delete_host = mock.Mock()
        self.client.api_mock = self.client.delete_host

    def test_delete_multiple(self):
        arglist = [
            't1',
            't2',
        ]
        verifylist = [
            ('host', ['t1', 't2'])
        ]
        # Verify cm is triggereg with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Set the response
        self.client.api_mock.side_effect = [{}, {}]

        # Trigger the action
        self.cmd.take_action(parsed_args)

        calls = [
            mock.call(host='t1', ignore_missing=False),
            mock.call(host='t2', ignore_missing=False)
        ]

        self.client.api_mock.assert_has_calls(calls)
        self.assertEqual(2, self.client.api_mock.call_count)


class TestListServer(fakes.TestDeH):

    _host = fakes.FakeHost.create_one()
    objects = fakes.FakeServer.create_multiple(3)

    columns = (
        'addresses', 'id', 'name', 'metadata', 'status', 'user_id'
    )

    data = []

    for s in objects:
        data.append(fakes.gen_data(s, columns))

    def setUp(self):
        super(TestListServer, self).setUp()

        self.cmd = host.ListServer(self.app, None)

        self.client.servers = mock.Mock()
        self.client.find_host = mock.Mock()
        self.client.api_mock = self.client.servers

    def test_default(self):
        arglist = [
            'zn',
        ]

        verifylist = [
            ('host', 'zn'),
        ]

        # Verify cm is triggereg with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Set the response
        self.client.api_mock.side_effect = [
            self.objects
        ]
        self.client.find_host.side_effect = [
            self.data
        ]

        # Trigger the action
        columns, data = self.cmd.take_action(parsed_args)

        self.client.find_host.assert_called_once_with(
            'zn',
            ignore_missing=False
        )

        self.client.api_mock.assert_called_once_with(
            host=self.data,
        )

        self.assertEqual(self.columns, columns)
        self.assertEqual(self.data, list(data))


class TestListHostType(fakes.TestDeH):

    objects = fakes.FakeHostType.create_multiple(3)

    columns = (
        'host_type', 'host_type_name'
    )

    data = []

    for s in objects:
        data.append(fakes.gen_data(s, columns))

    def setUp(self):
        super(TestListHostType, self).setUp()

        self.cmd = host.ListHostType(self.app, None)

        self.client.host_types = mock.Mock()
        self.client.api_mock = self.client.host_types

    def test_default(self):
        arglist = [
            'az',
        ]

        verifylist = [
            ('az', 'az'),
        ]

        # Verify cm is triggereg with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Set the response
        self.client.api_mock.side_effect = [
            self.objects
        ]

        # Trigger the action
        columns, data = self.cmd.take_action(parsed_args)

        self.client.api_mock.assert_called_once_with(
            'az'
        )

        self.assertEqual(self.columns, columns)
        self.assertEqual(self.data, list(data))



#!/usr/bin/env python3
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
#
"""Update a training job configuration."""
import openstack

openstack.enable_logging(True)
conn = openstack.connect(cloud="otc")
attr = {
    "config_desc": "This is config",
    "worker_server_num": 1,
    "app_url": "/usr/app/",
    "boot_file_url": "/usr/app/boot.py",
    "parameter": [
        {
            "label": "learning_rate",
            "value": 0.01,
        },
        {
            "key": "batch_size",
            "value": 32,
        },
    ],
    "spec_id": 1,
    "dataset_id": "38277e62-9e59-48f4-8d89-c8cf41622c24",
    "dataset_version_id": "2ff0d6ba-c480-45ae-be41-09a8369bfc90",
    "engine_id": 1,
    "train_url": "/usr/train/",
    "log_url": "/usr/log/",
}
config_name = "dads"
response = conn.modelartsv1.update_training_job_config(config_name, **attr)
print(response)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

"""
Managing policy types in the Cluster service.

For a full guide see
https://developer.openstack.org/sdks/python/openstacksdk/user/guides/cluster.html
"""


def list_policy_types(conn):
    print("List Policy Types:")

    for pt in conn.clustering.policy_types():
        print(pt.to_dict())


def get_policy_type(conn):
    print("Get Policy Type:")

    pt = conn.clustering.get_policy_type('senlin.policy.deletion-1.0')

    print(pt.to_dict())



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from openstack import resource
from openstack import exceptions


class TagSpec(resource.Resource):
    #: Key
    key = resource.Body('key')
    #: Value.
    values = resource.Body('values', type=list)


class Function(resource.Resource):
    resources_key = 'functions'
    base_path = '/fgs/functions'

    # Capabilities
    allow_create = True
    allow_fetch = True
    allow_commit = True
    allow_delete = True
    allow_list = True

    _query_mapping = resource.QueryParameters(
        'package_name', 'maxitems', 'marker'
    )

    # Properties
    func_name = resource.Body('func_name', type=str)
    package = resource.Body('package', type=str)
    runtime = resource.Body('runtime', type=str)
    timeout = resource.Body('timeout', type=int)
    handler = resource.Body('handler', type=str)
    depend_version_list = resource.Body('depend_version_list', type=list)
    func_vpc = resource.Body('func_vpc', type=dict)
    memory_size = resource.Body('memory_size', type=int)
    code_type = resource.Body('code_type', type=str)
    code_url = resource.Body('code_url', type=str)
    code_filename = resource.Body('code_filename', type=str)
    custom_image = resource.Body('custom_image', type=dict)
    user_data = resource.Body('user_data', type=str)
    encrypted_user_data = resource.Body('encrypted_user_data', type=str)
    xrole = resource.Body('xrole', type=str)
    app_xrole = resource.Body('app_xrole', type=str)
    description = resource.Body('description', type=str)
    func_code = resource.Body('func_code', type=dict)
    mount_config = resource.Body('mount_config', type=dict)
    initializer_handler = resource.Body('initializer_handler', type=str)
    initializer_timeout = resource.Body('initializer_timeout', type=int)
    pre_stop_handler = resource.Body('pre_stop_handler', type=str)
    pre_stop_timeout = resource.Body('pre_stop_timeout', type=int)
    enterprise_project_id = resource.Body('enterprise_project_id', type=str)
    type = resource.Body('type', type=str)
    log_config = resource.Body('log_config', type=dict)
    network_controller = resource.Body('network_controller', type=dict)
    is_stateful_function = resource.Body('is_stateful_function', type=bool)
    enable_dynamic_memory = resource.Body('enable_dynamic_memory', type=bool)

    # Attributes
    func_id = resource.Body('func_id', type=str)
    func_urn = resource.Body('func_urn', type=str)
    domain_id = resource.Body('domain_id', type=str)
    namespace = resource.Body('namespace', type=str)
    project_name = resource.Body('project_name', type=str)
    cpu = resource.Body('cpu', type=int)
    code_size = resource.Body('code_size', type=int)
    domain_names = resource.Body('domain_names', type=str)
    digest = resource.Body('digest', type=str)
    version = resource.Body('version', type=str)
    image_name = resource.Body('image_name', type=str)
    last_modified = resource.Body('last_modified', type=str)
    reserved_instance_count = resource.Body(
        'reserved_instance_count', type=int)
    strategy_config = resource.Body('strategy_config', type=dict)
    extend_config = resource.Body('extend_config', type=str)
    dependencies = resource.Body('dependencies', type=list)
    long_time = resource.Body('long_time', type=bool)
    log_group_id = resource.Body('log_group_id', type=str)
    log_stream_id = resource.Body('log_stream_id', type=str)
    enable_cloud_debug = resource.Body('enable_cloud_debug', type=str)
    is_bridge_function = resource.Body('is_bridge_function', type=bool)
    apig_route_enable = resource.Body('apig_route_enable', type=bool)
    heartbeat_handler = resource.Body('heartbeat_handler', type=str)
    enable_class_isolation = resource.Body('enable_class_isolation', type=bool)
    allow_ephemeral_storage = resource.Body(
        'allow_ephemeral_storage', type=bool)
    ephemeral_storage = resource.Body('ephemeral_storage', type=int)
    resource_id = resource.Body('resource_id', type=str)
    is_return_stream = resource.Body('is_return_stream', type=bool)
    enable_auth_in_header = resource.Body('enable_auth_in_header', type=bool)
    gpu_memory = resource.Body('gpu_memory', type=int)
    func_vpc_id = resource.Body('func_vpc_id', type=str)
    bind_bridge_func_urns = resource.Body('bind_bridge_funcUrns', type=list)
    reserved_instance_idle_mode = resource.Body(
        'reserved_instance_idle_mode', type=bool)
    gpu_type = resource.Body('gpu_type', type=str)
    tags = resource.Body('tags', type=list, list_type=TagSpec)

    def _delete_function(self, session, function):
        """Delete Function
        """
        url = self.base_path + f'/{function.func_urn.rpartition(":")[0]}'
        response = session.delete(url)
        exceptions.raise_from_response(response)
        return None

    def _get_function_code(self, session, function):
        """Get Function Code
        """
        url = self.base_path + f'/{function.func_urn.rpartition(":")[0]}/code'
        response = session.get(url)
        exceptions.raise_from_response(response)
        self._translate_response(response)
        return self

    def _get_function_metadata(self, session, function):
        """Get Function Metadata
        """
        urn = function.func_urn.rpartition(":")[0]
        url = self.base_path + f'/{urn}/config'
        response = session.get(url)
        exceptions.raise_from_response(response)
        self._translate_response(response)
        return self

    def _get_resource_tags(self, session, function):
        """Get Resource Tags
        """
        url = f'/functions/tags'
        response = session.get(url)
        exceptions.raise_from_response(response)
        self.tags = response.json()["tags"]
        return self

    def _create_resource_tags(self, session, function, tags):
        """Create Resource Tags
        """
        data = {"tags": tags}
        url = f'/functions/{function.func_urn.rpartition(":")[0]}/tags/create'
        response = session.post(url, json=data)
        exceptions.raise_from_response(response)
        return None

    def _delete_resource_tags(self, session, function, tags):
        """Delete Resource Tags
        """
        data = {"tags": tags}
        url = f'/functions/{function.func_urn.rpartition(":")[0]}/tags/delete'
        response = session.delete(url, json=data)
        exceptions.raise_from_response(response)
        return None

    def _update_pin_status(self, session, function):
        """Update Pin Status
        """
        urn = function.func_urn.rpartition(":")[0]
        url = self.base_path + f'/{urn}/collect/true'
        response = session.put(url)
        exceptions.raise_from_response(response)
        self._translate_response(response)
        return self

    def _update_function_code(self, session, function, **attrs):
        """Update Function Code
        """
        url = self.base_path + f'/{function.func_urn.rpartition(":")[0]}/code'
        response = session.put(url, json=attrs)
        exceptions.raise_from_response(response)
        self._translate_response(response)
        return self

    def _update_function_metadata(self, session, function, **attrs):
        """Update Function Metadata
        """
        urn = function.func_urn.rpartition(":")[0]
        url = self.base_path + f'/{urn}/config'
        response = session.put(url, json=attrs)
        exceptions.raise_from_response(response)
        self._translate_response(response)
        return self

    def _update_max_instances(self, session, function, number):
        """Update Function Instances Number
        """
        urn = function.func_urn.rpartition(":")[0]
        url = self.base_path + f'/{urn}/config-max-instance'
        response = session.put(url, json={'max_instance_num': number})
        exceptions.raise_from_response(response)
        self._translate_response(response)
        return self



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from openstack.tests.unit import base

from otcextensions.sdk.cts.v1 import tracker

FAKE_ID = "68d5745e-6af2-40e4-945d-fe449be00148"
EXAMPLE = {
    "bucket_name": "my_created_bucket",
    "tracker_name": "system",
    "detail": "noBucket",
    "file_prefix_name": "some_folder",
    "status": "disabled",
    "smn": {
        "is_support_smn": False,
        "topic_id": "urn:smn:regionId:someid:cts-test",
        "is_send_all_key_operation": False,
        "operations": ["delete", "create", "login"],
        "need_notify_user_list": ["user1", "user2"]
    }
}


class TestTracker(base.TestCase):

    def test_basic(self):
        sot = tracker.Tracker()

        self.assertEqual('/tracker', sot.base_path)

        self.assertTrue(sot.allow_fetch)
        self.assertTrue(sot.allow_create)
        self.assertTrue(sot.allow_delete)
        self.assertTrue(sot.allow_commit)

    def test_make_it(self):

        sot = tracker.Tracker(**EXAMPLE)
        self.assertEqual(EXAMPLE['bucket_name'], sot.bucket_name)
        self.assertEqual(EXAMPLE['tracker_name'], sot.name)
        self.assertEqual(EXAMPLE['detail'], sot.detail)
        self.assertEqual(EXAMPLE['file_prefix_name'], sot.file_prefix_name)
        self.assertEqual(EXAMPLE['status'], sot.status)
        self.assertEqual(EXAMPLE['smn']['is_support_smn'], sot.smn.enabled)
        self.assertEqual(EXAMPLE['smn']['topic_id'], sot.smn.topic_id)
        self.assertEqual(
            EXAMPLE['smn']['is_send_all_key_operation'],
            sot.smn.is_send_all_key_operation)
        self.assertEqual(EXAMPLE['smn']['operations'], sot.smn.operations)
        self.assertEqual(
            EXAMPLE['smn']['need_notify_user_list'],
            sot.smn.notify_users)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
#
import time

from openstack import exceptions
from openstack import proxy
from otcextensions.sdk.modelartsv1.v1 import builtin_model as _builtin_model
from otcextensions.sdk.modelartsv1.v1 import devenv as _devenv
from otcextensions.sdk.modelartsv1.v1 import job_engine as _job_engine
from otcextensions.sdk.modelartsv1.v1 import job_flavor as _job_flavor
from otcextensions.sdk.modelartsv1.v1 import model as _model
from otcextensions.sdk.modelartsv1.v1 import service as _service
from otcextensions.sdk.modelartsv1.v1 import \
    service_cluster as _service_cluster
from otcextensions.sdk.modelartsv1.v1 import service_event as _service_event
from otcextensions.sdk.modelartsv1.v1 import service_flavor as _service_flavor
from otcextensions.sdk.modelartsv1.v1 import service_log as _service_log
from otcextensions.sdk.modelartsv1.v1 import \
    service_monitor as _service_monitor
from otcextensions.sdk.modelartsv1.v1 import training_job as _training_job
from otcextensions.sdk.modelartsv1.v1 import \
    training_job_config as _training_job_config
from otcextensions.sdk.modelartsv1.v1 import \
    training_job_version as _training_job_version
from otcextensions.sdk.modelartsv1.v1 import \
    visualization_job as _visualization_job


class Proxy(proxy.Proxy):
    skip_discovery = True

    def __init__(self, session, *args, **kwargs):
        super(Proxy, self).__init__(session=session, *args, **kwargs)
        self.additional_headers = {
            "Accept": "application/json",
            "Content-type": "application/json",
        }

    # ======== BuiltIn Model Management ========
    def builtin_models(self, **params):
        """List all BuiltIn Models.

        :param dict params: Optional query parameters to be sent to limit
            the built-in models being returned.

        :returns: a generator of :class:`~otcextensions.sdk.modelartsv1.v1.
            built_in_model.BuiltInModel` instances.
        """
        return self._list(_builtin_model.BuiltInModel, **params)

    def find_builtin_model(self, name_or_id, ignore_missing=True):
        """Find a single built-in model.

        :param name_or_id: The name or ID of a built-in model
        :param bool ignore_missing: When set to ``False``
            :class:`~openstack.exceptions.ResourceNotFound` will be raised
            if the model does not exist.
            When set to ``True``, no exception will be set when attempting
            to find a nonexistent built-in model.

        :returns: One :class:`~otcextensions.sdk.modelartsv1.v1.
            built_in_model.BuiltInModel` or ``None``.
        """
        return self._find(
            _builtin_model.BuiltInModel,
            name_or_id,
            ignore_missing=ignore_missing,
        )

    # ======== Model Management ========

    def models(self, **params):
        """List all Models.

        :param dict params: Optional query parameters to be sent to limit
            the models being returned.

        :returns: a generator of
            :class:`~otcextensions.sdk.modelartsv1.v1.model.Model` instances.
        """
        params["paginated"] = False
        return self._list(_model.Model, **params)

    def create_model(self, **attrs):
        """Create a model from attributes.

        :param dict attrs: Keyword arguments which will be used to create
            a :class:`~otcextensions.sdk.modelartsv1.v1.models.Model`,
            comprised of the properties on the Model class.

        :returns: The results of model creation.
        :rtype: :class:`~otcextensions.modelartsv1.v1.model.Model`
        """
        return self._create(_model.Model, prepend_key=False, **attrs)

    def get_model(self, model):
        """Get details of a model by Id.

        :param model: model id or an instance of
            :class:`~otcextensions.sdk.modelartsv1.v1.model.Model`

        :returns: instance of
            :class:`~otcextensions.sdk.modelartsv1.v1.model.Model`
        """
        return self._get(_model.Model, model)

    def find_model(self, name_or_id, ignore_missing=True):
        """Find a single model.

        :param name_or_id: The name or ID of a model.
        :param bool ignore_missing: When set to ``False``
            :class:`~openstack.exceptions.ResourceNotFound` will be raised
            if the model does not exist.
            When set to ``True``, no exception will be set when attempting
            to find a nonexistent model.

        :returns:
            One :class:`~otcextensions.sdk.modelartsv1.v1.model.Model`
            or ``None``.
        """
        return self._find(
            _model.Model, name_or_id, ignore_missing=ignore_missing
        )

    def delete_model(self, model, ignore_missing=False):
        """Delete a model.

        :param model: model id or an instance of
            :class:`~otcextensions.sdk.modelartsv1.v1.model.Model`
        :param bool ignore_missing: When set to ``False``
            :class:`~openstack.exceptions.ResourceNotFound` will be raised when
            the model does not exist.
            When set to ``True``, no exception will be set when attempting to
            delete a nonexistent model.

        :returns: ``None``
        """
        return self._delete(_model.Model, model, ignore_missing=ignore_missing)

    # ======== DevEnviron Management ========

    def devenv_instances(self, **params):
        """List all Devenv Instances.

        :param dict params: Optional query parameters to be sent to limit
            the instances being returned.

        :returns: a generator of
            :class:`~otcextensions.sdk.modelartsv1.v1.devenv.Devenv` instances.
        """
        if not params.get("dev_env_type"):
            params.update(dev_env_type="Notebook")
        if params.get("limit"):
            params.update(paginated=False)
        return self._list(_devenv.Devenv, **params)

    def create_devenv_instance(self, **attrs):
        """Create a devenv instance from attributes.

        :param dict attrs: Keyword arguments which will be used to create
            a :class:`~otcextensions.sdk.modelartsv1.v1.devenv.Devenv`,
            comprised of the properties on the Devenv class.

        :returns: The results of devenv instance creation.
        :rtype: :class:`~otcextensions.sdk.modelartsv1.v1.devenv.Devenv`
        """
        return self._create(_devenv.Devenv, prepend_key=False, **attrs)

    def get_devenv_instance(self, instance):
        """Get details of a Devenv Instance.

        :param instance: devenv instance id or an instance of
            :class:`~otcextensions.sdk.modelartsv1.v1.devenv.Devenv`.

        :returns: instance of
            :class:`~otcextensions.sdk.modelartsv1.v1.devenv.Devenv`.
        """
        return self._get(_devenv.Devenv, instance)

    def find_devenv_instance(self, name_or_id, ignore_missing=False):
        """Find a single Devenv instance by name or Id.

        :param name_or_id: The name or ID of a Devenv instance.

        :param bool ignore_missing: When set to ``False``
            :class:`~openstack.exceptions.ResourceNotFound` will be raised
            if the devenv does not exist.
            When set to ``True``, no exception will be set when attempting
            to find a nonexistent devenv.

        :returns:
            One :class:`~otcextensions.sdk.modelartsv1.v1.devenv.Devenv`
            or ``None``
        """
        return self._find(
            _devenv.Devenv,
            name_or_id,
            ignore_missing=ignore_missing,
            de_type="Notebook",
        )

    def update_devenv_instance(self, instance, **attrs):
        """Update a Devenv Instance Configurations.

        :param instance: devenv instance id or an instance of
            :class:`~otcextensions.sdk.modelartsv1.v1.devenv.Devenv`
        :param dict attrs: Keyword arguments which will be used to update
            a :class:`~otcextensions.sdk.modelartsv1.v1.devenv.Devenv`,
            comprised of the properties on the Devenv class.

        """
        return self._update(_devenv.Devenv, instance, **attrs)

    def start_devenv_instance(self, instance):
        """Start a Devenv instance.

        :param instance: devenv instance id or an instance of
            :class:`~otcextensions.sdk.modelartsv1.v1.devenv.Devenv`

        :returns: instance of
            :class:`~otcextensions.sdk.modelartsv1.v1.devenv.Devenv`
        """
        devenv = self._get_resource(_devenv.Devenv, instance)
        return devenv.start(self)

    def stop_devenv_instance(self, instance):
        """Stop a Devenv instance.

        :param instance: devenv instance id or an instance of
            :class:`~otcextensions.sdk.modelartsv1.v1.devenv.Devenv`

        :returns: instance of
            :class:`~otcextensions.sdk.modelartsv1.v1.devenv.Devenv`
        """
        devenv = self._get_resource(_devenv.Devenv, instance)
        return devenv.stop(self)

    def delete_devenv_instance(self, instance, ignore_missing=False):
        """Delete a Devenv instance.

        :param instance: devenv instance id or an instance of
            :class:`~otcextensions.sdk.modelartsv1.v1.devenv.Devenv`.
        :param bool ignore_missing: When set to ``False``
            :class:`~openstack.exceptions.ResourceNotFound` will be raised when
            the group does not exist.
            When set to ``True``, no exception will be set when attempting to
            delete a nonexistent devenv.

        :returns: ``None``
        """
        return self._delete(
            _devenv.Devenv, instance, ignore_missing=ignore_missing
        )

    # Service Management

    def services(self, **params):
        """List all Services.

        :param dict params: Optional query parameters to be sent to limit
            the services being returned.

        :returns: a generator of
            :class:`~otcextensions.sdk.modelartsv1.v1.service.Service`
            instances
        """
        params["paginated"] = False
        return self._list(_service.Service, **params)

    def create_service(self, **attrs):
        """Deploy a modelarts service from attributes.

        :param dict attrs: Keyword arguments which will be used to create
            a :class:`~otcextensions.sdk.modelartsv1.v1.service.Service`,
            comprised of the properties on the Service class.

        :returns: The result of service creation.
        :rtype: :class:`~otcextensions.sdk.modelartsv1.v1.service.Service`
        """
        return self._create(_service.Service, **attrs)

    def delete_service(self, service, ignore_missing=False):
        """Delete a modelarts service.

        :param service: service id or an instance of
            :class:`~otcextensions.sdk.modelartsv1.v1.service.Service`
        :param bool ignore_missing: When set to ``False``
            :class:`~openstack.exceptions.ResourceNotFound` will be raised when
            the service does not exist.
            When set to ``True``, no exception will be set when attempting to
            delete a nonexistent service.

        :returns: ``None``
        """
        return self._delete(
            _service.Service, service, ignore_missing=ignore_missing
        )

    def get_service(self, service):
        """Get details of a modelarts service.

        :param service: service id or an instance of
            :class:`~otcextensions.sdk.modelartsv1.v1.service.Service`

        :returns: instance of
            :class:`~otcextensions.sdk.modelartsv1.v1.service.Service`
        """
        return self._get(_service.Service, service)

    def find_service(self, name_or_id, ignore_missing=True):
        """Find a single service

        :param name_or_id: The name or ID of a ModelArts service
        :param bool ignore_missing: When set to ``False``
            :class:`~openstack.exceptions.ResourceNotFound` will be raised
            if the service does not exist.
            When set to ``True``, no exception will be set when attempting
            to find a nonexistent service.

        :returns: instance of
            :class:`~otcextensions.sdk.modelartsv1.v1.service.Service`
        """
        return self._find(
            _service.Service, name_or_id, ignore_missing=ignore_missing
        )

    def update_service(self, service, **attrs):
        """Update a Service Configurations.

        :param service: service id or an instance of
            :class:`~otcextensions.sdk.modelartsv1.v1.service.Service`
        :param dict attrs: Keyword arguments which will be used to update
            a :class:`~otcextensions.sdk.modelartsv1.v1.service.Service`,
            comprised of the properties on the Service class.

        :returns: instance of
            :class:`~otcextensions.sdk.modelartsv1.v1.service.Service`
        """
        return self._update(_service.Service, service, **attrs)

    def start_service(self, service):
        """Start a Service.

        :param service: service id or an instance of
            :class:`~otcextensions.sdk.modelartsv1.v1.service.Service`

        :returns: instance of
            :class:`~otcextensions.sdk.modelartsv1.v1.service.Service`
        """
        return self._update(_service.Service, service, status="running")

    def stop_service(self, service):
        """Stop a Service.

        :param service: service id or an instance of
            :class:`~otcextensions.sdk.modelartsv1.v1.service.Service`

        :returns: instance of
            :class:`~otcextensions.sdk.modelartsv1.v1.service.Service`
        """
        return self._update(_service.Service, service, status="stopped")

    def wait_for_service(self, service, timeout=1200, wait=5):
        while timeout > 0:
            obj = self.get_service(service)
            status = obj.status.lower()
            if status == "deploying":
                pass
            elif status in ["running", "finished"]:
                return True
            else:
                raise exceptions.SDKException(obj.error_msg)
            timeout = timeout - wait
            time.sleep(wait)
        raise exceptions.SDKException(
            f"Wait Timed Out. service status is: {status}"
        )

    def service_logs(self, service, **params):
        """List update logs of a real-time service.

        :param service: service id or an instance of
            :class:`~otcextensions.sdk.modelartsv1.v1.service.Service`
        :param dict params: Optional query parameters to be sent to limit
            the service logs being returned.

        :returns: a generator of
            :class:`~otcextensions.sdk.modelartsv1.v1.service_log.ServiceLog`
            instances
        """
        service = self._get_resource(_service.Service, service)
        return self._list(
            _service_log.ServiceLog, service_id=service.id, **params
        )

    def service_events(self, service, **params):
        """List events logs of a service.

        :param service: service id or an instance of
            :class:`~otcextensions.sdk.modelartsv1.v1.service.Service`
        :param dict params: Optional query parameters to be sent to limit
            the service events being returned.

        :returns: a generator of
            :class:`~otcextensions.sdk.modelartsv1.v1.service_event.ServiceEvent`
            instances
        """
        service = self._get_resource(_service.Service, service)
        return self._list(
            _service_event.ServiceEvent,
            service_id=service.id,
            paginated=False,
            **params,
        )

    def service_monitor(self, service, **params):
        """List a service monitoring informations.

        :param service: service id or an instance of
            :class:`~otcextensions.sdk.modelartsv1.v1.service.Service`

        :returns: a generator of
            :class:`~otcextensions.sdk.modelartsv1.v1.service_monitor.ServiceMonitor`
            instances
        """
        service = self._get_resource(_service.Service, service)
        return self._list(
            _service_monitor.ServiceMonitor, service_id=service.id, **params
        )

    def service_flavors(self, **params):
        """List all flavors for a service deployment.

        :param dict params: Optional query parameters to be sent to limit
            the service flavors being returned.
        :returns: a generator of
            :class:`~otcextensions.sdk.modelartsv1.v1.service_flavor.ServiceFlavor`
            instances
        """
        return self._list(_service_flavor.ServiceFlavor, **params)

    def service_clusters(self, **params):
        """List all dedicated resource pools (clusters) available
        for a service deployment.

        :param dict params: Optional query parameters to be sent to limit
            the service clusters being returned.
        :returns: a generator of
            :class:`~otcextensions.sdk.modelartsv1.v1.service_cluster.ServiceCluster`
            instances
        """
        return self._list(_service_cluster.ServiceCluster, **params)

    # Training Job Management

    def training_jobs(self, **params):
        """List all training jobs.

        :param dict params: Optional query parameters to be sent to limit
            the training jobs being returned.

        :returns: a generator of
            :class:`~otcextensions.sdk.modelartsv1.v1.training_job.TrainingJob`
            instances
        """
        return self._list(_training_job.TrainingJob, **params)

    def create_training_job(self, **attrs):
        """Create a training job from attributes.

        :param dict attrs: Keyword arguments which will be used to create a
            :class:`~otcextensions.sdk.modelartsv1.v1.training_job.TrainingJob`,
            comprised of the properties on the TrainingJob class.

        :returns: The results of training job creation.
        :rtype:
            :class:`~otcextensions.sdk.modelartsv1.v1.training_job.TrainingJob`
        """
        return self._create(_training_job.TrainingJob, **attrs)

    def find_training_job(self, name_or_id, ignore_missing=True):
        """Find a single training job by name or Id.

        :param name_or_id: The name or ID of a modelarts training job
        :param bool ignore_missing: When set to ``False``
            :class:`~openstack.exceptions.ResourceNotFound` will be raised
            if the trainjob does not exist.
            When set to ``True``, no exception will be set when attempting
            to find a nonexistent trainjob.

        :returns: instance of
            :class:`~otcextensions.sdk.modelarts.v1.training_job.TrainingJob`
            or ``None``
        """
        return self._find(
            _training_job.TrainingJob,
            name_or_id,
            ignore_missing=ignore_missing,
        )

    def update_training_job(self, training_job, description):
        """Update training job description.

        :param training_job: training job id or an instance of
            :class:`~otcextensions.sdk.modelarts.v1.training_job.TrainingJob`
        :param description: Description of a training job.

        :returns: instance of
            :class:`~otcextensions.sdk.modelarts.v1.training_job.TrainingJob`
        """
        return self._update(
            _training_job.TrainingJob,
            training_job,
            job_desc=description,
        )

    def delete_training_job(self, training_job, ignore_missing=False):
        """Delete a training job

        :param training_job: training job id or an instance of
            :class:`~otcextensions.sdk.modelarts.v1.training_job.TrainingJob`
        :param bool ignore_missing: When set to ``False``
            :class:`~openstack.exceptions.ResourceNotFound` will be raised when
            the training_job does not exist.
            When set to ``True``, no exception will be set when attempting to
            delete a nonexistent training job.

        returns: ``None``
        """
        return self._delete(
            _training_job.TrainingJob,
            training_job,
            ignore_missing=ignore_missing,
        )

    def job_flavors(self, **params):
        """List all flavors available for running a job.

        :param dict params: Optional query parameters to be sent to limit
            the job flavors being returned.

        :returns: a generator of
            :class:`~otcextensions.sdk.modelartsv1.v1.job_flavor.JobFlavor`
            instances.
        """
        return self._list(_job_flavor.JobFlavor, **params)

    def job_engines(self, **params):
        """List all engines available for running a job.

        :param dict params: Optional query parameters to be sent to limit
            the job engines being returned.

        :returns: a generator of
            :class:`~otcextensions.sdk.modelartsv1.v1.job_engine.JobEngine`
            instances.
        """
        return self._list(_job_engine.JobEngine, **params)

    # Training Job Version Management

    def training_job_versions(self, training_job, **attrs):
        """List versions of a training job.

        :param training_job: training job id or an instance of
            :class:`~otcextensions.sdk.modelartsv1.v1.training_job.TrainingJob`
        :param dict params: Optional query parameters to be sent to limit
            the training job versions being returned.

        :returns: a generator of
            :class:`~otcextensions.sdk.modelartsv1.v1.
            training_job.TrainingJobVersion` instances.
        """
        training_job = self._get_resource(
            _training_job.TrainingJob, training_job
        )
        return self._list(
            _training_job_version.TrainingJobVersion,
            job_id=training_job.id,
            **attrs,
        )

    def get_training_job_version(self, training_job, version):
        """Get details of a training job by version id.

        :param training_job: training job id or an instance of
            :class:`~otcextensions.sdk.modelartsv1.v1.training_job.TrainingJob`
        :param version: training job version id or an instance of
            :class:`~otcextensions.sdk.modelartsv1.v1.
            training_job.TrainingJobVersion`

        :returns: instance of
            :class:`~otcextensions.sdk.modelartsv1.v1.
            training_job.TrainingJobVersion`
        """
        training_job = self._get_resource(
            _training_job.TrainingJob, training_job
        )
        return self._get(
            _training_job_version.TrainingJobVersion,
            version,
            job_id=training_job.id,
        )

    def create_training_job_version(self, training_job, **attrs):
        """Create a training job from attributes

        :param training_job: training job id or an instance of
            :class:`~otcextensions.sdk.modelartsv1.v1.training_job.TrainingJob`
        :param dict attrs: Keyword arguments which will be used to create
            a :class:`~otcextensions.sdk.modelartsv1.v1.trainjob.Trainjob`,
            comprised of the properties on the Trainjob class.

        :returns: The results of trainjobs creation
        :rtype: :class:`~otcextensions.sdk.modelartsv1.v1.
            training_job.TrainingJobVersion`
        """
        training_job = self._get_resource(
            _training_job.TrainingJob, training_job
        )
        return self._create(
            _training_job_version.TrainingJobVersion,
            job_id=training_job.id,
            **attrs,
        )

    def delete_training_job_version(
        self, training_job, version, ignore_missing=False
    ):
        """Delete a training job version.

        :param training_job: training job id or an instance of
            :class:`~otcextensions.sdk.modelartsv1.v1.training_job.TrainingJob`
        :param version: training job version id or an instance of
            :class:`~otcextensions.sdk.modelartsv1.v1.
            training_job.TrainingJobVersion`
        :param bool ignore_missing: When set to ``False``
            :class:`~openstack.exceptions.ResourceNotFound` will be raised when
            the training job version does not exist.
            When set to ``True``, no exception will be set when attempting to
            delete a nonexistent training job version.

        returns: ``None``
        """
        training_job = self._get_resource(
            _training_job.TrainingJob, training_job
        )
        return self._delete(
            _training_job_version.TrainingJobVersion,
            version,
            job_id=training_job.id,
            ignore_missing=ignore_missing,
        )

    def stop_training_job_version(self, training_job, version):
        """Stop a training job version.

        :param training_job: training job id or an instance of
            :class:`~otcextensions.sdk.modelartsv1.v1.training_job.TrainingJob`
        :param version: training job version id or an instance of
            :class:`~otcextensions.sdk.modelartsv1.v1.
            training_job.TrainingJobVersion`

        :rtype: :class:`~otcextensions.sdk.modelartsv1.v1.
            training_job.TrainingJobVersion`
        """
        training_job = self._get_resource(
            _training_job.TrainingJob, training_job
        )
        training_job_version = self._get_resource(
            _training_job_version.TrainingJobVersion,
            version,
            job_id=training_job.id,
        )
        return training_job_version.stop(self)

    # Training Job Config Management

    def training_job_configs(self, **params):
        """List all training job configurations.

        :param dict params: Optional query parameters to be sent to limit
            the training job configs being returned.

        :returns: a generator of
            (:class:`~otcextensions.sdk.modelartsv1.v1.
            training_job_config.TrainingjobConfig`) instances.
        """
        return self._list(_training_job_config.TrainingJobConfig, **params)

    def create_training_job_config(self, **attrs):
        """Create a Training Job Configuration from attributes.

        :param dict attrs: Keyword arguments which will be used to create a
            :class:`~otcextensions.sdk.modelartsv1.v1.
            training_job_config.TrainingjobConfig`,
            comprised of the properties on the TrainingJobConfig class.

        :returns: The results of Training Job Configuration creation.
        :rtype: :class:`~otcextensions.sdk.modelartsv1.v1.
            training_job.TrainingJobConfig`
        """
        return self._create(_training_job_config.TrainingJobConfig, **attrs)

    def get_training_job_config(self, name):
        """Get the Training Job Configuration by config name.

        :param name: Name of a training job configuration.

        :returns: instance of :class:`~otcextensions.sdk.modelartsv1.v1.
            training_job_config.TrainingJobConfig`.
        """
        return self._get(_training_job_config.TrainingJobConfig, name)

    def update_training_job_config(self, name, **attrs):
        """Update training job configuration.

        :param name: Name of a training job configuration.
        :param dict attrs: Keyword arguments which will be used to update a
            :class:`~otcextensions.sdk.modelartsv1.v1.
            training_job_config.TrainingJobConfig`,
            comprised of the properties on the class.

        :returns: instance of :class:`~otcextensions.sdk.modelartsv1.v1.
            training_job_config.TrainingJobConfig`.
        """
        return self._update(
            _training_job_config.TrainingJobConfig, name, **attrs
        )

    def delete_training_job_config(self, name, ignore_missing=False):
        """Delete Training Job Configuration.

        :param name: Name of a training job configuration.
        :param bool ignore_missing: When set to ``False``
            :class:`~openstack.exceptions.ResourceNotFound` will be raised when
            the config does not exist.
            When set to ``True``, no exception will be set when attempting to
            delete a nonexistent Training Job Configuration.

        returns: ``None``
        """
        return self._delete(
            _training_job_config.TrainingJobConfig,
            name,
            ignore_missing=ignore_missing,
        )

    # Visualization Job Management

    def visualization_jobs(self, **params):
        """List all Visualization Jobs.

        :param dict params: Optional query parameters to be sent to limit
            the visualization jobs  being returned.

        :returns: a generator of :class:`~otcextensions.sdk.modelartsv1.v1.
            visualization_job.VisualizationJob` instances.
        """
        return self._list(_visualization_job.VisualizationJob, **params)

    def create_visualization_job(self, **attrs):
        """Create a Visualization Job from attributes.

        :param dict attrs: Keyword arguments which will be used to create a
            :class:`~otcextensions.sdk.modelartsv1.v1.
            visualization_job.VisualizationJob`
            comprised of the properties on the Visualization Job class.

        :returns: The results of Visualization Job creation.

        :rtype:
            :class:`~otcextensions.sdk.modelartsv1.v1.
            visualization_job.VisualizationJob`.
        """
        return self._create(_visualization_job.VisualizationJob, **attrs)

    def get_visualization_job(self, visualization_job):
        """Get details of a Visualization Job.

        :param visualization_job: visualization job id or an instance of
            :class:`~otcextensions.sdk.modelartsv1.v1.
            visualization_job.VisualizationJob`.

        :returns: instance of
            :class:`~otcextensions.sdk.modelartsv1.v1.
            visualization_job.VisualizationJob`.
        """
        return self._get(
            _visualization_job.VisualizationJob, visualization_job
        )

    def update_visualization_job(self, visualization_job, description):
        """Update visualization job description.

        :param visualization_job: visualization job id or an instance of
            :class:`~otcextensions.sdk.modelartsv1.v1.
            visualization_job.VisualizationJob`.
        :param description: Description of a visualization job to update.

        :returns: instance of :class:`~otcextensions.sdk.modelartsv1.v1.
            visualization_job.VisualizationJob`
        """
        return self._update(
            _visualization_job.VisualizationJob,
            visualization_job,
            job_desc=description,
        )

    def delete_visualization_job(
        self, visualization_job, ignore_missing=False
    ):
        """Delete a Visualization Job

        :param visualization_job: visualization job id or an instance of
            :class:`~otcextensions.sdk.modelartsv1.v1.
            visualization_job.VisualizationJob`.
        :param bool ignore_missing: When set to ``False``
            :class:`~openstack.exceptions.ResourceNotFound` will be raised when
            the group does not exist.
            When set to ``True``, no exception will be set when attempting to
            delete a nonexistent Visualization Job.

        returns: ``None``
        """
        return self._delete(
            _visualization_job.VisualizationJob,
            visualization_job,
            ignore_missing=ignore_missing,
        )

    def stop_visualization_job(self, visualization_job):
        """Stop a Visualization Job.

        :param visualization_job: visualization job id or an instance of
            :class:`~otcextensions.sdk.modelarts.v1.
            visualization_job.VisualizationJob`.

        :returns: instance of :class:`~otcextensions.sdk.modelartsv1.v1.
            visualization_job.VisualizationJob`.
        """
        visualization_job = self._get_resource(
            _visualization_job.VisualizationJob, visualization_job
        )
        return visualization_job.stop(self)

    def restart_visualization_job(self, visualization_job):
        """Restart a Visualization Job.

        :param visualization_job: visualization job id or an instance of
            :class:`~otcextensions.sdk.modelarts.v1.
            visualization_job.VisualizationJob`.

        :returns: instance of
            :class:`~otcextensions.sdk.modelartsv1.v1.
            visualization_job.VisualizationJob`.
        """
        visualization_job = self._get_resource(
            _visualization_job.VisualizationJob, visualization_job
        )
        return visualization_job.restart(self)
